#ifndef	__GPSLOGGER_H__
#define	__GPSLOGGER_H__
//*******************************************************
//
//	GPS Logger
//
//	MCU:ATmega328p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.4
//				AVR-LIBC	1.6.7
//				binutils	2.20
//
//	2010.5.1	KAWAKAMI Yukio
//*******************************************************

#define		CR	13
#define		LF	10
#define		TAB	9
#define		ON	1
#define		OFF	0
//#define		TRUE	1
//#define		FALSE	0

#define		NOP		asm volatile("nop\n"::)
#define		WDR		asm volatile("wdr\n"::)
#define		SLEEP	asm volatile("sleep\n"::)

typedef	unsigned char	uchar;
typedef	unsigned int	uint;
typedef	unsigned long	ulong;
typedef unsigned short	ushort;

#define	CPU_CLOCK	8000000L

#define	R10_VALUE	150
#define	R11_VALUE	75

#define	SCK		5	// PORTB
#define	MISO	4
#define	MOSI	3
#define	CCS		2

#define	LRS		7	// PORTD
#define	LCS		6
#define	LRES	5
#define	POWER	4
#define	SW1		3
#define	PPS		2
#define	TXD		1
#define	RXD		0

#define	VOLT	6	// ADC6

typedef struct typ_GPS_DATA {
	char	status;
	char	lat_ew;
	char	lat_d;
	char	lat_m;
	int		lat_s;
	char	long_ns;
	int		long_d;
	char	long_m;
	int		long_s;
	int		altitude;
	int		year;
	char	month;
	char	day;
	char	hour;
	char	min;
	char	sec;
	char	satellites;
	int		speed;		// 0.1km/h単位
	int		direction;	// 0～360度
	int		hdop;		// HDOP * 10
} GPS_DATA;

void	job_serial(void);

void	write_log(char *str);
void	power_off(void);

char	get_str(uchar *source, char n, uchar *dest, char max);

void	GPS_command(PGM_P str);

void	SPI_out(uchar data);
uchar	SPI_in(void);
int		ADC_in(char ch);

void	wait_timer(uchar t);


#endif	// __GPSLOGGER_H__
