//*******************************************************
//
//	GPS Logger
//
//	シリアル通信関係
//
//	Compiler:	AVR-GCC		4.3.3
//				AVR-LIBC	1.6.2
//				binutils	2.18
//
//	2010.4.6	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>

#include	"GPSLogger.h"
#include	"com.h"

// シリアル受信バッファ
volatile unsigned char	Rbuff[RX_BUFF_SIZE];		// シリアル受信バッファ
volatile unsigned char	Rremain = 0;	// 受信バッファの残り数
unsigned char			Rin = 0;		// 入力位置
unsigned char			Rout = 0;		// 出力位置


#if TX_BUFF_SIZE > 0
// シリアル送信バッファ
volatile unsigned char	Sbuff[32];
volatile unsigned char	Sremain = 0;
unsigned char			Sin = 0;
unsigned char			Sout = 0;
#endif

//********************************************************************
//	シリアル 受信割り込み
//********************************************************************
ISR(USART_RX_vect){

	if (UCSR0A & ((1<<FE0)|(1<<DOR0)|(1<<UPE0))){	// 受信エラー
		unsigned char	dummy;
		dummy = UDR0;	// 読み捨て
	} else {
		Rbuff[Rin++] = UDR0;
		if (Rin >= sizeof(Rbuff)){
			Rin = 0;
		}
		Rremain++;
	}
}


#if TX_BUFF_SIZE > 0
//********************************************************************
//	シリアル送信データレジスタ空き割り込み
//********************************************************************
ISR(USART_UDRE_vect){

	if (Sremain){
		UDR0 = Sbuff[Sout++];
		if (Sout >= sizeof(Sbuff))	Sout = 0;
		--Sremain;
	}

	if (Sremain == 0){
		UCSR0B &= ~(1<<UDRIE0);	// 送信データレジスタ空き割り込み禁止
	}
}
#endif


//********************************************************************
//	シリアルポート初期化
//********************************************************************
void init_com(uint baud){

	cli();	// 割り込み禁止

	UBRR0H = ((CPU_CLOCK/16/(ulong)baud) - 1) >> 8;
	UBRR0L = ((CPU_CLOCK/16/(ulong)baud) - 1);

	UCSR0A = 0;
	UCSR0B = (1<<RXCIE0)|(1<<RXEN0)|(1<<TXEN0);	// 送受信許可、受信完了割り込み許可

	//UCSR0C = (1<<USBS0)|(1<<UCSZ01)|(1<<UCSZ00);	// 送信Stop 2、8bit non
	UCSR0C = (1<<UCSZ01)|(1<<UCSZ00);	// Stop1 8bit non

	Rin = Rout = Rremain = 0;
#if TX_BUFF_SIZE > 0
	Sin = Sout = Sremain = 0;
#endif

	sei();	// 割り込み許可
}


//********************************************************************
//	受信バッファに入っているデータの byte数取得
//********************************************************************
uchar check_com(void){

	return Rremain;
}


//********************************************************************
//	シリアルポート受信バッファから 1byte読み込み
//********************************************************************
unsigned char read_com(void){
	unsigned char	data;

	// 受信データが無ければ来るまで待つ
	while(!Rremain){
		WDR;
		SLEEP;
	}

	// 受信バッファからデータ読み出し
	data = Rbuff[Rout++];
	if (Rout >= sizeof(Rbuff))	Rout = 0;
	cli();
	--Rremain;
	sei();
	
	return data;
}


//********************************************************************
//	シリアルポート 1byte送信
//********************************************************************
void write_com(unsigned char data){

#if TX_BUFF_SIZE > 0
	// 送信バッファがいっぱいなら待つ
	while(Sremain >= sizeof(Sbuff)){
		WDR;
		SLEEP;
	}

	// データを送信バッファに積む
	Sbuff[Sin++] = data;
	if (Sin >= sizeof(Sbuff))	Sin = 0;
	cli();
	Sremain++;
	sei();

	// 送信データレジスタ空き割り込み許可
	UCSR0B |= (1<<UDRIE0);
#else
	// 送信データレジスタ空きフラグチェック
	while(!(UCSR0A & (1<<UDRE0)));
	UDR0 = data;
#endif
}


//********************************************************************
//	RAM上文字列送信
//********************************************************************
void write_str(uchar *str){
	unsigned char	d;

	while((d = *str++)){
		write_com(d);
	}
}


//********************************************************************
//	固定文字列送信
//********************************************************************
void write_str_P(PGM_P str){
	unsigned char	d;

	while((d = pgm_read_byte(str++))){
		write_com(d);
	}
}


