//*******************************************************
//
//	GPS Logger
//
//	101 x 64 モノクロ液晶表示
//
//	MCU:ATmega328p
//		efuse:0xFD	BOD 2.7V
//		hfuse:0xD1	EEPROM消去保護
//		lfuse:0xE2	内蔵CR発振 8MHz
//
//	Compiler:	AVR-GCC		4.3.3
//				AVR-LIBC	1.6.2
//				binutils	2.18
//
//	2010.4.9	KAWAKAMI Yukio
//*******************************************************

#include	<avr/io.h>
#include	<avr/interrupt.h>
#include	<avr/pgmspace.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"GPSLogger.h"
#include	"LCDfont.h"
#include	"NumFont.h"
#include	"lcd.h"

uchar	CurX, CurY;
#ifdef USE_LCD_BUFF
uchar	LcdBuff[101*64/8];	// LCDバッファ 808bytes
#endif


//********************************************************************
//	時刻表示用文字列出力(数字とコロンのみ)
//********************************************************************
void print_lerge_num(char x, char y, char *str){
	char	data;
	char	w, h;
	PGM_P	p;

	while((data = *str++)){
		if (isdigit(data)){
			p = (PGM_P)&NumData[data - '0'];
		} else {
			p = (PGM_P)&NumData[10];
		}
		for (h = 0; h < 16/8; h++){
			LCD_set(x, y/8+h);
			for (w = 0; w < 10; w++){
				LCD_data(pgm_read_byte(p++));
			}
		}
		x += 10;
	}
}


//********************************************************************
//	ASCIIフォント出力(6 x 8ドット)
//********************************************************************
void putch(char ch){

	if (ch == CR){
		CurX = 0;
	} else if (ch == LF){
		CurX = 0;
		if (CurY < LCD_HEIGHT)	CurY += 8;
	} else {
		if ((ch < ' ')||(ch > 0x7F)){
			ch = '.';
		}
		if (CurX > (LCD_WIDTH - 6)){
			CurX = 0;
			if (CurY < LCD_HEIGHT)	CurY += 8;
		}
		if (CurY < (LCD_HEIGHT - 7)){
			PGM_P	p;
			uchar	i;
#ifdef USE_LCD_BUFF
			uchar	*bp;
#endif
			uchar	d;

			if ((CurY & 7) == 0){
				// マスク処理不要

				LCD_set(CurX, CurY/8);
#ifdef USE_LCD_BUFF
				bp = &LcdBuff[(CurY/8)*LCD_WIDTH+CurX];
#endif
				p = (PGM_P)&(AsciiFont[((int)(ch - 0x20)) * 5]);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++);
					LCD_data(d);
#ifdef USE_LCD_BUFF
					*bp++ = d;
#endif
				}
				LCD_data(0);
#ifdef USE_LCD_BUFF
				*bp = 0;
#endif
			} else {
#ifdef USE_LCD_BUFF
				// マスク処理必要
				uchar	mask;
				uchar	shift;

				shift = CurY & 7;

				mask = ~(0xFF << shift);
				bp = &LcdBuff[((int)CurY/8)*LCD_WIDTH+CurX];
				p = (PGM_P)&(AsciiFont[((int)(ch - 0x20)) * 5]);
				LCD_set(CurX, CurY/8);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++) << shift;
					d |= (*bp & mask);
					LCD_data(d);
					*bp++ = d;
				}
				d = (*bp & mask);
				LCD_data(d);
				*bp = d;

				bp += LCD_WIDTH - 5;
				
				mask = 0xFF << shift;
				p -= 5;
				LCD_set(CurX, CurY/8+1);
				for (i = 0; i < 5; i++){
					d = (uchar)pgm_read_byte(p++) >> (8 - shift);
					d |= (*bp & mask);
					LCD_data(d);
					*bp++ = d;
				}
				d = (*bp & mask);
				LCD_data(d);
				*bp = d;
#endif
			}
			CurX += 6;
		}
	}
}


//*******************************************************
//	文字列表示
//*******************************************************
void print(char *str){
	char	c;

	while((c = *str++)){
		putch(c);
	}
}


//*******************************************************
//	固定文字列表示
//*******************************************************
void print_P(PGM_P str){
	char	c;

	while((c = pgm_read_byte(str++))){
		putch(c);
	}
}


//*********************************************************
//	LCD RAMの書き込み開始位置セット
//	xは1ピクセル単位、yは8ピクセル単位
//*********************************************************
void LCD_set(uchar x, uchar y){

	PORTD &= ~((1<<LCS)|(1<<LRS));
	SPI_out(0x10 + (x >> 4));
	SPI_out(x & 0x0F);
	SPI_out(0xB0 + y);
	PORTD |= (1<<LCS);
}


//*********************************************************
//	LCD消去
//*********************************************************
void clear_LCD(void){
	uchar	x, y;
	uint	i;

	// 画面クリア
	i = 0;
	for (y = 0; y < 8; y++){
		LCD_set(0, y);
		for (x = 0; x < 101; x++){
			LCD_data(0);
#ifdef USE_LCD_BUFF
			LcdBuff[i++] = 0;
#endif
		}
	}

	CurX = CurY = 0;
}


//*********************************************************
//	LCD初期化
//*********************************************************
void LCD_init(void){
	uint	i;

	PORTD &= ~(1<<LRES);	// LCDリセット
	NOP;
	NOP;
	PORTD |= (1<<LRES);	// LCDリセット解除

	LCD_command(0xE2);		// reset
	LCD_command(0xA0);		// normal direction
	LCD_command(0xC0);		// normal direction
	LCD_command(0xA2);		// bias
	LCD_command(0x81);		// reference voltage mode
	LCD_command(0x20);

	for (i = 0; i < 2500; i++){	// 適当
		NOP;
	}

	LCD_command(0x2F);		// Power control 内部レギュレータ等、ON
	LCD_command(0x25);		// コントラスト設定抵抗値 1+Rb/Ra = 5.5

	for (i = 0; i < 2500; i++){	// 適当
		NOP;
	}

	clear_LCD();

	LCD_command(0xAF);		// display on

	LCD_contrast(45);		// コントラスト初期値
}


//*********************************************************
//	LCDコントラスト設定
//*********************************************************
void LCD_contrast(uchar contrast){

	if(contrast < 20) contrast = 20;
	if(contrast > 50) contrast = 50;

	PORTD &= ~((1<<LCS)|(1<<LRS));
	SPI_out(0x81);
	SPI_out(contrast);
	PORTD |= (1<<LCS);
}



//********************************************************************
//	LCD コマンド出力
//********************************************************************
void LCD_command(uchar com){

	PORTD &= ~((1<<LCS)|(1<<LRS));
	SPI_out(com);
	PORTD |= (1<<LCS);
}


//********************************************************************
//	LCD データ出力
//********************************************************************
void LCD_data(uchar data){

	PORTD &= ~(1<<LCS);
	PORTD |= (1<<LRS);
	SPI_out(data);
	PORTD |= (1<<LCS);
}


